//
//  FMDBDataAccess.m
//  BMXMDB
//
//  Created by Nick Hayday on 21/02/2012.
//  Copyright (c) 2012 __MyCompanyName__. All rights reserved.
//

#import "FMDBDataAccess.h"

@implementation FMDBDataAccess

- (NSMutableArray *) getFilms {
    
    NSMutableArray *films = [[NSMutableArray alloc] init];
    
    FMDatabase *db = [FMDatabase databaseWithPath:[Utility getDatabasePath]];
    
    [db open];
    
    FMResultSet *results = [db executeQuery:@"SELECT * FROM film order by (name collate nocase)"];
    
    while([results next]) {
        
        Film *film = [[Film alloc] init];
        
        film.filmID = [results intForColumn:@"id"];
        film.filmName = [results stringForColumn:@"name"];
        film.format_web = [results boolForColumn:@"format_web"];
        film.format_dvd = [results boolForColumn:@"format_dvd"];
        film.format_vhs = [results boolForColumn:@"format_vhs"];
        film.format_blu = [results boolForColumn:@"format_blu"];
        film.country = [results stringForColumn:@"country"];
        film.runtime = [results stringForColumn:@"runtime"];
        film.notes = [results stringForColumn:@"notes"];
        film.year = [results intForColumn:@"release_year"];
        film.companies = [results stringForColumn:@"companies"];
               

        
        [films addObject:film];
        
    }
    
    [db close];
    
    return films;
    
}

- (NSMutableArray *) getFavouriteFilms {
    
    NSMutableArray *films = [[NSMutableArray alloc] init];
    
    FMDatabase *db = [FMDatabase databaseWithPath:[Utility getDatabasePath]];
    
    [db open];
    
    FMResultSet *results = [db executeQuery:@"select * from film where id in (select film_id from favourites)"];
    
    while([results next]) {
        
        Film *film = [[Film alloc] init];
        
        film.filmID = [results intForColumn:@"id"];
        film.filmName = [results stringForColumn:@"name"];
        film.format_web = [results boolForColumn:@"format_web"];
        film.format_dvd = [results boolForColumn:@"format_dvd"];
        film.format_vhs = [results boolForColumn:@"format_vhs"];
        film.format_blu = [results boolForColumn:@"format_blu"];
        film.country = [results stringForColumn:@"country"];
        film.runtime = [results stringForColumn:@"runtime"];
        film.notes = [results stringForColumn:@"notes"];
        film.year = [results intForColumn:@"release_year"];
        film.companies = [results stringForColumn:@"companies"];
        
        
        
        [films addObject:film];
        
    }
    
    [db close];
    
    return films;
    
}


- (NSMutableArray *) getPeople {
    
    NSMutableArray *people = [[NSMutableArray alloc] init];
    
    FMDatabase *db = [FMDatabase databaseWithPath:[Utility getDatabasePath]];
    
    [db open];
    
    FMResultSet *results = [db executeQuery:@"SELECT * FROM people order by (name collate nocase)"];
    
    while([results next]) {
        
        People *person = [[People alloc] init];
        
        person.peopleID = [results intForColumn:@"id"];
        person.peopleName = [results stringForColumn:@"name"];
        
        [people addObject:person];
        
    }
    
    [db close];
    
    return people;
    
}

- (NSMutableArray *) getCompanies {
    
    
    NSMutableArray *companies = [[NSMutableArray alloc] init];
    
    FMDatabase *db = [FMDatabase databaseWithPath:[Utility getDatabasePath]];
    
    [db open];
    
    FMResultSet *results = [db executeQuery:@"SELECT * FROM company order by (name collate nocase)"];
    
    while ([results next]) {
        
        Company *company = [[Company alloc] init];
        
        company.companyID = [results intForColumn:@"id"];
        company.companyName = [results stringForColumn:@"name"];
        
        [companies addObject:company];
        
    }
    
    [db close];
    
    return companies;
    
}

- (NSMutableArray *) getSectionedFilms {
    
    NSMutableArray *sectionedFilms = [[NSMutableArray alloc] init];
    
    NSArray *queryArray = [NSArray arrayWithObjects:
                            @"A", @"B", @"C", @"D", @"E", @"F", @"G", @"H", @"I", @"J", @"K", @"L",
                            @"M", @"N", @"O", @"P", @"Q", @"R", @"S", @"T", @"U", @"V", @"W", @"X", @"Y", @"Z", nil];
    
    id a;
    
    for (a in queryArray) {
        
        [sectionedFilms addObject:[self getFilms:a]];
        
    }
    
    [sectionedFilms addObject:[self getNumberedFilms]];
                                       
    return sectionedFilms;
}

- (NSMutableArray *) getFilms:(NSString *)queryString {
    
    NSMutableArray *queryResults = [[NSMutableArray alloc] init];
    
    FMDatabase *db = [FMDatabase databaseWithPath:[Utility getDatabasePath]];
    
    [db open];
    
    FMResultSet *results = [db executeQuery:@"SELECT * FROM film where name like ? order by (name collate nocase)",[NSString stringWithFormat:@"%@%%", queryString]];
    
        while([results next]) {
            
            Film *film = [[Film alloc] init];
            
            film.filmID = [results intForColumn:@"id"];
            film.filmName = [results stringForColumn:@"name"];
            film.format_web = [results boolForColumn:@"format_web"];
            film.format_dvd = [results boolForColumn:@"format_dvd"];
            film.format_vhs = [results boolForColumn:@"format_vhs"];
            film.format_blu = [results boolForColumn:@"format_blu"];
            film.country = [results stringForColumn:@"country"];
            film.runtime = [results stringForColumn:@"runtime"];
            film.notes = [results stringForColumn:@"notes"];
            film.year = [results intForColumn:@"release_year"];
            film.companies = [results stringForColumn:@"companies"];

            
            [queryResults addObject:film];
            
        }
    
    
    [db close];
    
    return queryResults;
    
}

- (NSMutableArray *) getNumberedFilms {
    
    NSMutableArray *queryResults = [[NSMutableArray alloc] init];
    
    FMDatabase *db = [FMDatabase databaseWithPath:[Utility getDatabasePath]];
    
    [db open];
    
    FMResultSet *results = [db executeQuery:@"SELECT *,substr(name,1,1) + 1 as numbers FROM film where numbers != 1 order by name"];
    
    while([results next]) {
        
        Film *film = [[Film alloc] init];
        
        film.filmID = [results intForColumn:@"id"];
        film.filmName = [results stringForColumn:@"name"];
        film.format_web = [results boolForColumn:@"format_web"];
        film.format_dvd = [results boolForColumn:@"format_dvd"];
        film.format_vhs = [results boolForColumn:@"format_vhs"];
        film.format_blu = [results boolForColumn:@"format_blu"];
        film.country = [results stringForColumn:@"country"];
        film.runtime = [results stringForColumn:@"runtime"];
        film.notes = [results stringForColumn:@"notes"];
        film.year = [results intForColumn:@"release_year"];
        film.companies = [results stringForColumn:@"companies"];

        
        [queryResults addObject:film];
        
    }
    
    
    [db close];
    
    return queryResults;    
    
}

- (NSMutableArray *) getNumberedCompanies {
    
    NSMutableArray *queryResults = [[NSMutableArray alloc] init];
    
    FMDatabase *db = [FMDatabase databaseWithPath:[Utility getDatabasePath]];
    
    [db open];
    
    FMResultSet *results = [db executeQuery:@"SELECT *,substr(name,1,1) + 1 as numbers FROM company where numbers != 1 order by name"];
    
    while ([results next]) {
        
        Company *company = [[Company alloc] init];
        
        company.companyID = [results intForColumn:@"id"];
        company.companyName = [results stringForColumn:@"name"];
        
        [queryResults addObject:company];
        
    }
    
    [db close];
    
    return queryResults;
    
    
    
}

- (NSMutableArray *) getCompanies:(NSString *)queryString {
    
    NSMutableArray *queryResults = [[NSMutableArray alloc] init];
    
    FMDatabase *db = [FMDatabase databaseWithPath:[Utility getDatabasePath]];
    
    [db open];
    
    FMResultSet *results = [db executeQuery:@"SELECT * FROM company where name like ? order by (name collate nocase)", [NSString stringWithFormat:@"%@%%",queryString]];
    
    while ([results next]) {
        
        Company *company = [[Company alloc] init];
        
        company.companyID = [results intForColumn:@"id"];
        company.companyName = [results stringForColumn:@"name"];
        
        [queryResults addObject:company];
        
    }
    
    [db close];
    
    return queryResults;
    
}

- (NSMutableArray *) getSectionedPeople {
    
    NSMutableArray *sectionedPeople = [[NSMutableArray alloc] init];
    
    NSArray *queryArray = [NSArray arrayWithObjects:
                           @"A", @"B", @"C", @"D", @"E", @"F", @"G", @"H", @"I", @"J", @"K", @"L",
                           @"M", @"N", @"O", @"P", @"Q", @"R", @"S", @"T", @"U", @"V", @"W", @"X", @"Y", @"Z", @"#", nil];
    
    id a;
    
    for (a in queryArray) {
        
        [sectionedPeople addObject:[self getPeople:a]];
        
    }
    
    return sectionedPeople;
    
    
    
}

- (NSMutableArray *) getSectionedCompanies {
    
    NSMutableArray *sectionedCompanies = [[NSMutableArray alloc] init];
    
    NSArray *queryArray = [NSArray arrayWithObjects:
                           @"A", @"B", @"C", @"D", @"E", @"F", @"G", @"H", @"I", @"J", @"K", @"L",
                           @"M", @"N", @"O", @"P", @"Q", @"R", @"S", @"T", @"U", @"V", @"W", @"X", @"Y", @"Z", nil];
    
    id a;
    
    for (a in queryArray) {
        
        [sectionedCompanies addObject:[self getCompanies:a]];
    }
    
    [sectionedCompanies addObject:[self getNumberedCompanies]];
    
    return sectionedCompanies;
    
}

- (NSMutableArray *) getPeople:(NSString *)queryString {
    
    NSMutableArray *queryResults = [[NSMutableArray alloc] init];
    
    FMDatabase *db = [FMDatabase databaseWithPath:[Utility getDatabasePath]];
    
    [db open];
    
    FMResultSet *results = [db executeQuery:@"SELECT * FROM people where name like ? order by (name collate nocase)",[NSString stringWithFormat:@"%@%%", queryString]];
    
    while([results next]) {
        
        People *people = [[People alloc] init];
        
        people.peopleID = [results intForColumn:@"id"];
        people.peopleName = [results stringForColumn:@"name"];
        
        [queryResults addObject:people];
        
    }
    
    
    [db close];
    
    return queryResults;
    
}

- (NSMutableArray *) getPersonsFilms:(NSInteger)personID {
    
    NSMutableArray *personsFilms = [[NSMutableArray alloc] init];
    NSMutableArray *riderFilms = [[NSMutableArray alloc] init];
    NSMutableArray *editorsFilms = [[NSMutableArray alloc] init];
    NSMutableArray *filmersFilms = [[NSMutableArray alloc] init];
    
    FMDatabase *db = [FMDatabase databaseWithPath:[Utility getDatabasePath]];
    
    [db open];
    
    //may want to filter out secions not mobile for riders
    
    FMResultSet *results = [db executeQuery:@"SELECT DISTINCT film.id, film.name, film.format_web, film.format_dvd, film.format_vhs, film.format_blu, film.country, film.runtime, film.notes, film.release_year, film.companies from film where id in (SELECT DISTINCT section.film_id FROM riders inner join section on riders.section_id = section.id where riders.person_id = ?) order by release_year",[NSNumber numberWithInt:personID]];
    
    while([results next]) {
        
        Film *film = [[Film alloc] init];
        
        film.filmID = [results intForColumn:@"id"];
        film.filmName = [results stringForColumn:@"name"];
        film.format_web = [results boolForColumn:@"format_web"];
        film.format_dvd = [results boolForColumn:@"format_dvd"];
        film.format_vhs = [results boolForColumn:@"format_vhs"];
        film.format_blu = [results boolForColumn:@"format_blu"];
        film.country = [results stringForColumn:@"country"];
        film.runtime = [results stringForColumn:@"runtime"];
        film.notes = [results stringForColumn:@"notes"];
        film.year = [results intForColumn:@"release_year"];
        film.companies = [results stringForColumn:@"companies"];
        
        [riderFilms addObject:film];
        
    }
    
    FMResultSet *results2 = [db executeQuery:@"select DISTINCT film.id, film.name, film.format_web, film.format_dvd, film.format_vhs, film.format_blu, film.country, film.runtime, film.notes, film.release_year, film.companies from film where id in (select distinct film_id from editors inner join film on editors.film_id = film.id where editors.person_id = ?) order by release_year",[NSNumber numberWithInt:personID]];
    
    while ([results2 next]) {
        
        Film *film = [[Film alloc] init];
        
        film.filmID = [results2 intForColumn:@"id"];
        film.filmName = [results2 stringForColumn:@"name"];
        film.format_web = [results2 boolForColumn:@"format_web"];
        film.format_dvd = [results2 boolForColumn:@"format_dvd"];
        film.format_vhs = [results2 boolForColumn:@"format_vhs"];
        film.format_blu = [results2 boolForColumn:@"format_blu"];
        film.country = [results2 stringForColumn:@"country"];
        film.runtime = [results2 stringForColumn:@"runtime"];
        film.notes = [results2 stringForColumn:@"notes"];
        film.year = [results2 intForColumn:@"release_year"];
        film.companies = [results2 stringForColumn:@"companies"];
        
                
        [editorsFilms addObject:film];
        
        
    }
    
    FMResultSet *results3 = [db executeQuery:@"select DISTINCT film.id, film.name, film.format_web, film.format_dvd, film.format_vhs, film.format_blu, film.country, film.runtime, film.notes, film.release_year, film.companies from film where id in (select distinct film_id from filmers inner join film on filmers.film_id = film.id where filmers.person_id = ?) order by release_year",[NSNumber numberWithInt:personID]];
    
    while ([results3 next]) {
        
        Film *film = [[Film alloc] init];
        
        film.filmID = [results3 intForColumn:@"id"];
        film.filmName = [results3 stringForColumn:@"name"];
        film.format_web = [results3 boolForColumn:@"format_web"];
        film.format_dvd = [results3 boolForColumn:@"format_dvd"];
        film.format_vhs = [results3 boolForColumn:@"format_vhs"];
        film.format_blu = [results3 boolForColumn:@"format_blu"];
        film.country = [results3 stringForColumn:@"country"];
        film.runtime = [results3 stringForColumn:@"runtime"];
        film.notes = [results3 stringForColumn:@"notes"];
        film.year = [results3 intForColumn:@"release_year"];
        film.companies = [results3 stringForColumn:@"companies"];
        
                
        [filmersFilms addObject:film];
        
        
    }
    
    
    [db close];
    
    [personsFilms addObject:riderFilms];
    [personsFilms addObject:editorsFilms];
    [personsFilms addObject:filmersFilms];
    
    
    return personsFilms;
    
}

- (NSMutableArray *) getCompaniesFilms:(NSInteger)companyID {
    
    NSMutableArray *companiesFilms = [[NSMutableArray alloc] init];
    
    FMDatabase *db = [FMDatabase databaseWithPath:[Utility getDatabasePath]];
    
    [db open];
    
    FMResultSet *results = [db executeQuery:@"select DISTINCT film.id, film.name, film.format_web, film.format_dvd, film.format_vhs, film.format_blu, film.country, film.runtime, film.notes, film.release_year, film.companies from film where id in (select distinct film_id from filmcompany inner join film on filmcompany.film_id = film.id where filmcompany.company_id = ?) order by release_year", [NSNumber numberWithInt:companyID]];
    
    while ([results next]) {
        
        Film *film = [[Film alloc] init];
        
        film.filmID = [results intForColumn:@"id"];
        film.filmName = [results stringForColumn:@"name"];
        film.format_web = [results boolForColumn:@"format_web"];
        film.format_dvd = [results boolForColumn:@"format_dvd"];
        film.format_vhs = [results boolForColumn:@"format_vhs"];
        film.format_blu = [results boolForColumn:@"format_blu"];
        film.country = [results stringForColumn:@"country"];
        film.runtime = [results stringForColumn:@"runtime"];
        film.notes = [results stringForColumn:@"notes"];
        film.year = [results intForColumn:@"release_year"];
        film.companies = [results stringForColumn:@"companies"];
        
        
        
        [companiesFilms addObject:film];
    }
    
    [db close];
    
    return companiesFilms;
    
}

- (NSMutableArray *) getFilmsCompanies:(NSInteger)filmID{
    
    NSMutableArray *filmsCompanies = [[NSMutableArray alloc] init];
    
    FMDatabase *db = [FMDatabase databaseWithPath:[Utility getDatabasePath]];
    
    [db open];
    
    FMResultSet *results = [db executeQuery:@"select distinct company.id, company.name from company where id in (select distinct company_id from filmcompany inner join company on filmcompany.company_id = company.id where film_id = ?)", [NSNumber numberWithInt:filmID]];
    
    while ([results next]) {
        
        Company *comp = [[Company alloc] init];
        
        comp.companyID = [results intForColumn:@"id"];
        comp.companyName = [results stringForColumn:@"name"];
        
        [filmsCompanies addObject:comp];
        
    }
    
    [db close];
    
    return filmsCompanies;
    
}

- (NSMutableArray *) getFilmsSections:(NSInteger)filmID {
    
    NSMutableArray *filmsSections = [[NSMutableArray alloc] init];
    
    FMDatabase *db = [FMDatabase databaseWithPath:[Utility getDatabasePath]];
    
    [db open];
    
    FMResultSet *results = [db executeQuery:@"select distinct id, film_id, name, url, notes, mobile from section where film_id = ?", [NSNumber numberWithInt:filmID]];
    
    while ([results next]) {
        
        FilmSections *filmSection = [[FilmSections alloc] init];
        
        filmSection.sectionID = [results intForColumn:@"id"];
        filmSection.filmID = [results intForColumn:@"film_id"];
        filmSection.sectionName = [results stringForColumn:@"name"];
        filmSection.url = [results stringForColumn:@"url"];
        filmSection.notes = [results stringForColumn:@"notes"];
        filmSection.mobile = [results boolForColumn:@"mobile"];
        
        [filmsSections addObject:filmSection];
        
    }
    
    [db close];
    
    return filmsSections;
    
}

- (FilmSections *) getFilmSection:(NSInteger)sectionID {
    
    FMDatabase *db = [FMDatabase databaseWithPath:[Utility getDatabasePath]];
    
    [db open];
    
    FMResultSet *results = [db executeQuery:@"select distinct id, film_id, name, url, notes from section where id = ?", [NSNumber numberWithInt:sectionID]];
    
    FilmSections *filmSection = [[FilmSections alloc] init];
    
    while ([results next]) {
        
        filmSection.sectionID = [results intForColumn:@"id"];
        filmSection.filmID = [results intForColumn:@"film_id"];
        filmSection.sectionName = [results stringForColumn:@"name"];
        filmSection.url = [results stringForColumn:@"url"];
        filmSection.notes = [results stringForColumn:@"notes"];
        filmSection.mobile = [results boolForColumn:@"mobile"];
    }
    
    [db close];
    
    return filmSection;
    
}

- (NSMutableArray *) getSectionsRiders:(NSInteger)sectionID {
    
//    NSLog(@"Started");
    
    NSMutableArray *sectionsRiders = [[NSMutableArray alloc] init];
    
    FMDatabase *db = [FMDatabase databaseWithPath:[Utility getDatabasePath]];
    
    [db open];
    
    FMResultSet *results = [db executeQuery:@"SELECT DISTINCT people.id, people.name from people where id in (SELECT DISTINCT riders.person_id FROM riders inner join section on riders.person_id = people.id where riders.section_id = ?)", [NSNumber numberWithInt:sectionID]];
    
    
    while ([results next]) {
     
        People *person = [[People alloc] init];
        
        person.peopleID = [results intForColumn:@"id"];
        person.peopleName = [results stringForColumn:@"name"];
        
        [sectionsRiders addObject:person];
        
        
    }
    
    [db close];
//    
//    NSLog(@"Stopped");
                            
    return sectionsRiders;
}

- (NSMutableArray *) getSectionsSongs:(NSInteger)sectionID {
    NSMutableArray *sectionsSongs = [[NSMutableArray alloc] init];
    
    FMDatabase *db = [FMDatabase databaseWithPath:[Utility getDatabasePath]];
    
    [db open];
    
    FMResultSet *results = [db executeQuery:@"SELECT DISTINCT songlist.id, songlist.name, songlist.artist from songlist where id in (SELECT DISTINCT song.songlist_id FROM song inner join section on song.songlist_id = songlist.id where song.section_id = ?)", [NSNumber numberWithInt:sectionID]];
    
    
    while ([results next]) {
        
        Song *s = [[Song alloc] init];
        
        s.songID = [results intForColumn:@"id"];
        
        s.songName = [results stringForColumn:@"name"];
        s.songArtist = [results stringForColumn:@"artist"];
        
        [sectionsSongs addObject:s];
        
    }
    
    [db close];
    
    return sectionsSongs;
}
    


@end
