//
//  HomeViewController.m
//  BMXMDB
//
//  Created by Nick Hayday on 21/02/2012.
//  Copyright (c) 2012 __MyCompanyName__. All rights reserved.
//

#import "HomeViewController.h"
#import "Reachability.h"
#import "MBProgressHUD.h"


@implementation HomeViewController

@synthesize lastReviewRefresh, favourites, items;


- (void)loadView {
    [super loadView];
    
    [[UIApplication sharedApplication]
     setNetworkActivityIndicatorVisible:YES];
    
    
    
    dispatch_queue_t jsonParsingQueue = dispatch_queue_create("jsonParsingQueue", NULL);
    
    dispatch_async(jsonParsingQueue, ^{
        
        
        [self fetchEntries];
        [self loadFavourites];
        [self turnOffRefresh];
        [self.tableView reloadData];
        [self hideActivityView];
        // once this is done, if you need to you can call
        // some code on a main thread (delegates, notifications, UI updates...)
                dispatch_async(dispatch_get_main_queue(), ^{
                    [[UIApplication sharedApplication]
                     setNetworkActivityIndicatorVisible:NO];
                    
                });
    });
    
    
    dispatch_release(jsonParsingQueue);
    
    
}

- (void)didReceiveMemoryWarning
{
    [super didReceiveMemoryWarning];
    // Release any cached data, images, etc that aren't in use.
}

#pragma mark - View lifecycle

- (void)viewDidLoad
{
    [super viewDidLoad];
        
    self.navigationController.navigationBar.barStyle=UIBarStyleBlackOpaque;
	// Do any additional setup after loading the view, typically from a nib.
    
    
    [self showActivityView];
    
        
    NSDate *d = [NSDate date];
    
    lastReviewRefresh = d;
    
    
    
}

- (void)viewWillAppear:(BOOL)animated
{
    [self.tableView deselectRowAtIndexPath:[self.tableView indexPathForSelectedRow] animated:animated];

}

- (void)loadFavourites
{
    favourites = [[NSMutableArray alloc] init];
    
    FMDBDataAccess *db = [[FMDBDataAccess alloc] init];
    
    favourites = [db getFavouriteFilms];

}

- (void)viewDidUnload
{
    [super viewDidUnload];

}

- (NSString *)tableView:(UITableView *)tableView titleForHeaderInSection:(NSInteger)section {
    
    NSArray *sectionArray = [NSArray arrayWithObjects:
                            @"Latest", @"Our Favourites", nil];
    
    
    return [sectionArray objectAtIndex:section];
    
}


- (NSInteger)tableView:(UITableView *)tableView 
 numberOfRowsInSection:(NSInteger)section
{
    if (section == 0){
        return [items count];
    } else {
        return [favourites count];
    }
        
}

- (UITableViewCell *)tableView:(UITableView *)tableView 
         cellForRowAtIndexPath:(NSIndexPath *)indexPath 
{
    
    static NSString *CellIdentifier1 = @"Cell1";
    static NSString *CellIdentifier2 = @"Cell2";
    
    if ([indexPath section] == 0){
	
        HomeTableCellView *cell = (HomeTableCellView *)[tableView dequeueReusableCellWithIdentifier:CellIdentifier1];
	if(cell == nil) {
		[[NSBundle mainBundle] loadNibNamed:@"HomeTableCellView" owner:self options:nil];
		cell = tblCell;
	}
            
        Item *item = [items objectAtIndex:[indexPath row]];
        
        [cell setTitleText:[NSString stringWithFormat:@"%@",item.title]];
        
        NSString *rawPost = [NSString stringWithFormat:@"%@",item.post];
        
        NSString *strippedPost = [rawPost stripHtml];
        
        [cell setDetailText:[NSString stringWithFormat:@"%@",strippedPost]];
    
    return cell;
        
    } else {
        
        
        HomeFilmTableCellView *cell = (HomeFilmTableCellView *)[tableView dequeueReusableCellWithIdentifier:CellIdentifier2];
        if(cell == nil) {
            [[NSBundle mainBundle] loadNibNamed:@"HomeFilmTableCellView" owner:self options:nil];
            cell = filmTblCell;
        }
        
        Film *film = [favourites objectAtIndex:[indexPath row]];
        
        [cell setTitleText:[NSString stringWithFormat:@"%@",film.filmName]];
        
        if (film.year != 0) {
            [cell  setYearText:[NSString stringWithFormat:@"Year:%d", film.year]];
            
        } else {
            [cell setYearText:[NSString stringWithFormat:@"Year:(Unknown)"]];
        }
        
        [cell setCover:[NSString stringWithFormat:@"%d",film.filmID]];
        
        [cell setCompanyText:film.companies];
        
        return cell;
        
    }
    
}

- (void)tableView:(UITableView *)tableView didSelectRowAtIndexPath:(NSIndexPath *)indexPath
{
    
    if ([indexPath section] == 0) {
    
    HomeDetailViewController *detail = [self.storyboard instantiateViewControllerWithIdentifier:@"HomeDetail"];
        
        Item *item = [items objectAtIndex:[indexPath row]];
        
        detail.item = item;
        
    [self.navigationController pushViewController:detail animated:YES];
        
    } else {
        
        
        FilmsDetailController *detail = [self.storyboard instantiateViewControllerWithIdentifier:@"FilmDetail"];
        
        Film *c = [favourites objectAtIndex:indexPath.row];
        
        detail.film = c;
        
        [self.navigationController pushViewController:detail animated:YES];
        
    }
        
}



- (void)fetchEntries
{
//    [[UIApplication sharedApplication]
//     setNetworkActivityIndicatorVisible:YES];
  
    NSString *urlString = [NSString stringWithFormat:@"http://bmxmdb.com/posts.json"];
//    NSString *urlString = [NSString stringWithFormat:@"http://127.0.0.1:3000/posts.json"];
    
    NSURL *url = [NSURL URLWithString:urlString];
    
    NSData *data = [NSData dataWithContentsOfURL:url];
    
    NSError *error;
    
    if (data != nil) {

        self.items = [[NSMutableArray alloc] init];
    
    NSMutableArray *json = (NSMutableArray*)[NSJSONSerialization JSONObjectWithData:data options:kNilOptions error:&error];
    
    
    
        for (NSDictionary *key in json)
        {
        
            Item *item = [[Item alloc] init];
    
            NSDictionary *temp = [key valueForKey:@"post"];
        
            [item setTitle:[temp valueForKey:@"title"]];
        
            [item setPost:[temp valueForKey:@"post"]];
        
            [item setSection_id:[temp valueForKey:@"section_id"]];
        
            [items addObject:item];
        
        }

    
    }
    
    
}

- (void)reloadTableViewDataSource
{
    
//    NSLog(@"%@", @"Did I get here?");
    
    [[UIApplication sharedApplication]
     setNetworkActivityIndicatorVisible:YES];
    
    [self fetchEntries];
        
    [self.tableView reloadData];
    
    [self turnOffRefresh];
    
    [[UIApplication sharedApplication]
     setNetworkActivityIndicatorVisible:NO];


	    
}

-(void)reachabilityChanged:(NSNotification*)note
{
    Reachability * reach = [note object];
    
    if([reach isReachable])
    {
        
    }
    else
    {
        [super dataSourceDidFinishLoadingNewData];
    }
    
    
}

- (NSInteger)numberOfSectionsInTableView:(UITableView *)tableView
{
    return 2;
}




- (void)turnOffRefresh 
{
   [super dataSourceDidFinishLoadingNewData];
    
    lastReviewRefresh = [NSDate date];
    
    refreshHeaderView.lastUpdatedDate = lastReviewRefresh;
}

-(void)showActivityView {
    if (overlayController == nil) {
        overlayController = [[ActivityOverlayViewController alloc] initWithFrame:self.view.bounds];
        
    }
    
    [self.navigationController.view addSubview:overlayController.view];

}

-(void)hideActivityView {
//    [overlayController.view removeFromSuperView];
    
    [overlayController.view removeFromSuperview];
}



@end
